export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);

        this.addInput("nPeriod", InputType.int, 9);
        this.addInput("factor_1", InputType.float, 0.6666666);
        this.addInput("factor_2", InputType.float, 0.3333333);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            percentK : this.addBuffer(),
            percentD : this.addBuffer(),
            percentJ : this.addBuffer(),
        };

        this.arr = {
            RSV : [],
        };

        this.addLevel(0);
        this.addLevel(20);
        this.addLevel(80);
        this.addLevel(100);
    }

    onInit() {
        this.buffers.percentK
            .setShape(Shape.Line)
            .setLabel("%K")
            .setColor(Color.Red);

        this.buffers.percentD
            .setShape(Shape.Line)
            .setLabel("%D")
            .setColor(Color.Blue);

        this.buffers.percentJ
            .setShape(Shape.Line)
            .setLabel("%J")
            .setColor(Color.Green);
    }

    async onUpdate() {
        const {nPeriod, factor_1, factor_2, PriceType} = this.getInputs(),
            {percentK, percentD, percentJ} = this.buffers,
            {
                High,
                Low,
                Close,
            } = await Bar.load([
                Bar.Mode.High,
                Bar.Mode.Low,
                Bar.Mode.Close,
            ], PriceType),
            barSize = Close.length;

        let i, k, num, Ln, Hn, Cn,
            counted_bars = Bar.counted();

        i = barSize - counted_bars - 1,
        num = barSize - nPeriod;

        while (i >= 0) {
            Cn = Close[i];
            Ln = Close[i];
            Hn = Close[i];

            for (k = 0; k < nPeriod; k++) {
                if (Ln > Low[i + k]) {
                    Ln = Low[i + k];
                }
                if (Hn < High[i + k]){
                    Hn = High[i + k];
                }
            }
            if (Hn !== Ln) {
                this.arr.RSV[i] = (Cn - Ln) / (Hn - Ln) * 100;
            } else {
                this.arr.RSV[i] = 50;
            }
            if (i >= num) {
                percentK.set(i, factor_1 * 50 + factor_2 * this.arr.RSV[i]);
                percentD.set(i, factor_1 * 50 + factor_2 * percentK.get(i));
            } else {
                percentK.set(i, factor_1 * percentK.get(i + 1) + factor_2 * this.arr.RSV[i]);
                percentD.set(i, factor_1 * percentD.get(i + 1) + factor_2 * percentK.get(i));
            }
            percentJ.set(i, 3 * percentD.get(i) - 2 * percentK.get(i));
            i--;
        }
    }
}